@@ -1,4 +1,4 @@
-static const char rcsid[] = "#(@) $Id: queue.c,v 1.4 2002/03/12 18:08:18 danda Exp $";
+static const char rcsid[] = "#(@) $Id: queue.c,v 1.7 2008/04/16 13:03:15 robincornelius Exp $";
 
 /* 
  * Date last modified: Jan 2001
@@ -17,7 +17,7 @@ static const char rcsid[] = "#(@) $Id: queue.c,v 1.4 2002/03/12 18:08:18 danda E
  *  Disclaimer: This code is released to the public domain.
  *
  *  Description:
- *      Generic double ended queue (Deque pronounced DEK) for handling
+ *      Generic double ended xmlrpc_queue (Deque pronounced DEK) for handling
  *      any data types, with sorting.
  *
  *      By use of various functions in this module the caller
@@ -25,22 +25,22 @@ static const char rcsid[] = "#(@) $Id: queue.c,v 1.4 2002/03/12 18:08:18 danda E
  *      sorted lists, indexed lists.  All lists are dynamic.
  *
  *      It is the responsibility of the caller to malloc and free
- *      memory for insertion into the queue. A pointer to the object
+ *      memory for insertion into the xmlrpc_queue. A pointer to the object
  *      is used so that not only can any data be used but various kinds
- *      of data can be pushed on the same queue if one so wished e.g.
+ *      of data can be pushed on the same xmlrpc_queue if one so wished e.g.
  *      various length string literals mixed with pointers to structures
  *      or integers etc.
  *
  *  Enhancements:
  *      A future improvement would be the option of multiple "cursors"
- *      so that multiple locations could occur in the one queue to allow
- *      placemarkers and additional flexibility.  Perhaps even use queue
+ *      so that multiple locations could occur in the one xmlrpc_queue to allow
+ *      placemarkers and additional flexibility.  Perhaps even use xmlrpc_queue
  *      itself to have a list of cursors.
  *
  * Usage:
  *
- *          /x init queue x/
- *          queue  q;
+ *          /x init xmlrpc_queue x/
+ *          xmlrpc_queue  q;
  *          Q_Init(&q);
  *
  *      To create a stack :
@@ -82,7 +82,7 @@ static const char rcsid[] = "#(@) $Id: queue.c,v 1.4 2002/03/12 18:08:18 danda E
  *
  *          if (Q_Find(&q, &mydata1, MyFunction))
  *          {
- *              /x found it, queue cursor now at correct record x/
+ *              /x found it, xmlrpc_queue cursor now at correct record x/
  *              /x can retrieve with x/
  *              data_ptr = Q_Get(&q);
  *
@@ -97,14 +97,13 @@ static const char rcsid[] = "#(@) $Id: queue.c,v 1.4 2002/03/12 18:08:18 danda E
  *
  ****************************************************************/
 
-
 #include <stdlib.h>
 #include "queue.h"
 
 
 static void QuickSort(void *list[], int low, int high,
                       int (*Comp)(const void *, const void *));
-static int  Q_BSearch(queue *q, void *key,
+static int  Q_BSearch(xmlrpc_queue *q, void *key,
                       int (*Comp)(const void *, const void *));
 
 /* The index: a pointer to pointers */
@@ -117,16 +116,16 @@ static  datanode    **posn_index;
  *
  ** function    : Q_Init
  *
- ** purpose     : Initialise queue object and pointers.
+ ** purpose     : Initialise xmlrpc_queue object and pointers.
  *
- ** parameters  : 'queue' pointer.
+ ** parameters  : 'xmlrpc_queue' pointer.
  *
  ** returns     : True_ if init successful else False_
  *
  ** comments    :
  ***/
 
-int Q_Init(queue  *q)
+int Q_Init(xmlrpc_queue  *q)
 {
    if(q) {
       q->head = q->tail = NULL;
@@ -142,9 +141,9 @@ int Q_Init(queue  *q)
  *
  ** function    : Q_AtHead
  *
- ** purpose     : tests if cursor is at head of queue
+ ** purpose     : tests if cursor is at head of xmlrpc_queue
  *
- ** parameters  : 'queue' pointer.
+ ** parameters  : 'xmlrpc_queue' pointer.
  *
  ** returns     : boolean - True_ is at head else False_
  *
@@ -152,7 +151,7 @@ int Q_Init(queue  *q)
  *
  ***/
 
-int Q_AtHead(queue *q)
+int Q_AtHead(xmlrpc_queue *q)
 {
    return(q && q->cursor == q->head);
 }
@@ -162,9 +161,9 @@ int Q_AtHead(queue *q)
  *
  ** function    : Q_AtTail
  *
- ** purpose     : boolean test if cursor at tail of queue
+ ** purpose     : boolean test if cursor at tail of xmlrpc_queue
  *
- ** parameters  : 'queue' pointer to test.
+ ** parameters  : 'xmlrpc_queue' pointer to test.
  *
  ** returns     : True_ or False_
  *
@@ -172,7 +171,7 @@ int Q_AtHead(queue *q)
  *
  ***/
 
-int Q_AtTail(queue *q)
+int Q_AtTail(xmlrpc_queue *q)
 {
    return(q && q->cursor == q->tail);
 }
@@ -182,17 +181,17 @@ int Q_AtTail(queue *q)
  *
  ** function    : Q_IsEmpty
  *
- ** purpose     : test if queue has nothing in it.
+ ** purpose     : test if xmlrpc_queue has nothing in it.
  *
- ** parameters  : 'queue' pointer
+ ** parameters  : 'xmlrpc_queue' pointer
  *
- ** returns     : True_ if IsEmpty queue, else False_
+ ** returns     : True_ if IsEmpty xmlrpc_queue, else False_
  *
  ** comments    :
  *
  ***/
 
-inline int Q_IsEmpty(queue *q)
+inline int Q_IsEmpty(xmlrpc_queue *q)
 {
    return(!q || q->size == 0);
 }
@@ -201,9 +200,9 @@ inline int Q_IsEmpty(queue *q)
  *
  ** function    : Q_Size
  *
- ** purpose     : return the number of elements in the queue
+ ** purpose     : return the number of elements in the xmlrpc_queue
  *
- ** parameters  : queue pointer
+ ** parameters  : xmlrpc_queue pointer
  *
  ** returns     : number of elements
  *
@@ -211,7 +210,7 @@ inline int Q_IsEmpty(queue *q)
  *
  ***/
 
-int Q_Size(queue *q)
+int Q_Size(xmlrpc_queue *q)
 {
    return q ? q->size : 0;
 }
@@ -221,17 +220,17 @@ int Q_Size(queue *q)
  *
  ** function    : Q_Head
  *
- ** purpose     : position queue cursor to first element (head) of queue.
+ ** purpose     : position xmlrpc_queue cursor to first element (head) of xmlrpc_queue.
  *
- ** parameters  : 'queue' pointer
+ ** parameters  : 'xmlrpc_queue' pointer
  *
- ** returns     : pointer to data at head. If queue is IsEmpty returns NULL
+ ** returns     : pointer to data at head. If xmlrpc_queue is IsEmpty returns NULL
  *
  ** comments    :
  *
  ***/
 
-void *Q_Head(queue *q)
+void *Q_Head(xmlrpc_queue *q)
 {
    if(Q_IsEmpty(q))
       return NULL;
@@ -246,17 +245,17 @@ void *Q_Head(queue *q)
  *
  ** function    : Q_Tail
  *
- ** purpose     : locate cursor at tail of queue.
+ ** purpose     : locate cursor at tail of xmlrpc_queue.
  *
- ** parameters  : 'queue' pointer
+ ** parameters  : 'xmlrpc_queue' pointer
  *
- ** returns     : pointer to data at tail , if queue IsEmpty returns NULL
+ ** returns     : pointer to data at tail , if xmlrpc_queue IsEmpty returns NULL
  *
  ** comments    :
  *
  ***/
 
-void *Q_Tail(queue *q)
+void *Q_Tail(xmlrpc_queue *q)
 {
    if(Q_IsEmpty(q))
       return NULL;
@@ -271,9 +270,9 @@ void *Q_Tail(queue *q)
  *
  ** function    : Q_PushHead
  *
- ** purpose     : put a data pointer at the head of the queue
+ ** purpose     : put a data pointer at the head of the xmlrpc_queue
  *
- ** parameters  : 'queue' pointer, void pointer to the data.
+ ** parameters  : 'xmlrpc_queue' pointer, void pointer to the data.
  *
  ** returns     : True_ if success else False_ if unable to push data.
  *
@@ -281,7 +280,7 @@ void *Q_Tail(queue *q)
  *
  ***/
 
-int Q_PushHead(queue *q, void *d)
+int Q_PushHead(xmlrpc_queue *q, void *d)
 {
    if(q && d) {
       node    *n;
@@ -323,9 +322,9 @@ int Q_PushHead(queue *q, void *d)
  *
  ** function    : Q_PushTail
  *
- ** purpose     : put a data element pointer at the tail of the queue
+ ** purpose     : put a data element pointer at the tail of the xmlrpc_queue
  *
- ** parameters  : queue pointer, pointer to the data
+ ** parameters  : xmlrpc_queue pointer, pointer to the data
  *
  ** returns     : True_ if data pushed, False_ if data not inserted.
  *
@@ -333,7 +332,7 @@ int Q_PushHead(queue *q, void *d)
  *
  ***/
 
-int Q_PushTail(queue *q, void *d)
+int Q_PushTail(xmlrpc_queue *q, void *d)
 {
    if(q && d) {
       node        *p;
@@ -375,17 +374,17 @@ int Q_PushTail(queue *q, void *d)
  ** function    : Q_PopHead
  *
  ** purpose     : remove and return the top element at the head of the
- *                queue.
+ *                xmlrpc_queue.
  *
- ** parameters  : queue pointer
+ ** parameters  : xmlrpc_queue pointer
  *
- ** returns     : pointer to data element or NULL if queue is IsEmpty.
+ ** returns     : pointer to data element or NULL if xmlrpc_queue is IsEmpty.
  *
  ** comments    :
  *
  ***/
 
-void *Q_PopHead(queue *q)
+void *Q_PopHead(xmlrpc_queue *q)
 {
    datanode    *n;
    void        *d;
@@ -417,18 +416,18 @@ void *Q_PopHead(queue *q)
  *
  ** function    : Q_PopTail
  *
- ** purpose     : remove element from tail of queue and return data.
+ ** purpose     : remove element from tail of xmlrpc_queue and return data.
  *
- ** parameters  : queue pointer
+ ** parameters  : xmlrpc_queue pointer
  *
- ** returns     : pointer to data element that was at tail. NULL if queue
+ ** returns     : pointer to data element that was at tail. NULL if xmlrpc_queue
  *                IsEmpty.
  *
  ** comments    :
  *
  ***/
 
-void *Q_PopTail(queue *q)
+void *Q_PopTail(xmlrpc_queue *q)
 {
    datanode    *p;
    void        *d;
@@ -460,19 +459,19 @@ void *Q_PopTail(queue *q)
  *
  ** function    : Q_Next
  *
- ** purpose     : Move to the next element in the queue without popping
+ ** purpose     : Move to the next element in the xmlrpc_queue without popping
  *
- ** parameters  : queue pointer.
+ ** parameters  : xmlrpc_queue pointer.
  *
  ** returns     : pointer to data element of new element or NULL if end
- *                of the queue.
+ *                of the xmlrpc_queue.
  *
  ** comments    : This uses the cursor for the current position. Q_Next
- *                only moves in the direction from the head of the queue
+ *                only moves in the direction from the head of the xmlrpc_queue
  *                to the tail.
  ***/
 
-void *Q_Next(queue *q)
+void *Q_Next(xmlrpc_queue *q)
 {
    if(!q)
       return NULL;
@@ -492,17 +491,17 @@ void *Q_Next(queue *q)
  ** function    : Q_Previous
  *
  ** purpose     : Opposite of Q_Next. Move to next element closer to the
- *                head of the queue.
+ *                head of the xmlrpc_queue.
  *
- ** parameters  : pointer to queue
+ ** parameters  : pointer to xmlrpc_queue
  *
- ** returns     : pointer to data of new element else NULL if queue IsEmpty
+ ** returns     : pointer to data of new element else NULL if xmlrpc_queue IsEmpty
  *
- ** comments    : Makes cursor move towards the head of the queue.
+ ** comments    : Makes cursor move towards the head of the xmlrpc_queue.
  *
  ***/
 
-void *Q_Previous(queue *q)
+void *Q_Previous(xmlrpc_queue *q)
 {
    if(!q)
       return NULL;
@@ -516,7 +515,7 @@ void *Q_Previous(queue *q)
 }
 
 
-void *Q_Iter_Del(queue *q, q_iter iter)
+void *Q_Iter_Del(xmlrpc_queue *q, q_iter iter)
 {
    void        *d;
    datanode    *n, *p;
@@ -552,7 +551,7 @@ void *Q_Iter_Del(queue *q, q_iter iter)
 
 
    if (n != NULL) {
-	n->prev = p;
+    n->prev = p;
    }
 
    q->size--;
@@ -568,20 +567,20 @@ void *Q_Iter_Del(queue *q, q_iter iter)
  *
  ** function    : Q_DelCur
  *
- ** purpose     : Delete the current queue element as pointed to by
+ ** purpose     : Delete the current xmlrpc_queue element as pointed to by
  *                the cursor.
  *
- ** parameters  : queue pointer
+ ** parameters  : xmlrpc_queue pointer
  *
  ** returns     : pointer to data element.
  *
  ** comments    : WARNING! It is the responsibility of the caller to
- *                free any memory. Queue cannot distinguish between
+ *                free any memory. xmlrpc_queue cannot distinguish between
  *                pointers to literals and malloced memory.
  *
  ***/
 
-void *Q_DelCur(queue* q) {
+void *Q_DelCur(xmlrpc_queue* q) {
    if(q) {
       return Q_Iter_Del(q, (q_iter)q->cursor);
    }
@@ -593,19 +592,19 @@ void *Q_DelCur(queue* q) {
  *
  ** function    : Q_Destroy
  *
- ** purpose     : Free all queue resources
+ ** purpose     : Free all xmlrpc_queue resources
  *
- ** parameters  : queue pointer
+ ** parameters  : xmlrpc_queue pointer
  *
  ** returns     : null.
  *
  ** comments    : WARNING! It is the responsibility of the caller to
- *                free any memory. Queue cannot distinguish between
+ *                free any memory. xmlrpc_queue cannot distinguish between
  *                pointers to literals and malloced memory.
  *
  ***/
 
-void Q_Destroy(queue *q)
+void Q_Destroy(xmlrpc_queue *q)
 {
    while(!Q_IsEmpty(q)) {
       Q_PopHead(q);
@@ -619,7 +618,7 @@ void Q_Destroy(queue *q)
  *
  ** purpose     : get the pointer to the data at the cursor location
  *
- ** parameters  : queue pointer
+ ** parameters  : xmlrpc_queue pointer
  *
  ** returns     : data element pointer
  *
@@ -627,7 +626,7 @@ void Q_Destroy(queue *q)
  *
  ***/
 
-void *Q_Get(queue *q)
+void *Q_Get(xmlrpc_queue *q)
 {
    if(!q)
       return NULL;
@@ -645,7 +644,7 @@ void *Q_Get(queue *q)
  *
  ** purpose     : replace pointer to data with new pointer to data.
  *
- ** parameters  : queue pointer, data pointer
+ ** parameters  : xmlrpc_queue pointer, data pointer
  *
  ** returns     : boolean- True_ if successful, False_ if cursor at NULL
  *
@@ -653,7 +652,7 @@ void *Q_Get(queue *q)
  *
  ***/
 
-int Q_Put(queue *q, void *data)
+int Q_Put(xmlrpc_queue *q, void *data)
 {
    if(q && data) {
       if(q->cursor == NULL)
@@ -670,28 +669,28 @@ int Q_Put(queue *q, void *data)
  *
  ** function    : Q_Find
  *
- ** purpose     : Linear search of queue for match with key in *data
+ ** purpose     : Linear search of xmlrpc_queue for match with key in *data
  *
- ** parameters  : queue pointer q, data pointer with data containing key
+ ** parameters  : xmlrpc_queue pointer q, data pointer with data containing key
  *                comparison function here called Comp.
  *
- ** returns     : True_ if found , False_ if not in queue.
+ ** returns     : True_ if found , False_ if not in xmlrpc_queue.
  *
  ** comments    : Useful for small queues that are constantly changing
  *                and would otherwise need constant sorting with the
  *                Q_Seek function.
  *                For description of Comp see Q_Sort.
- *                Queue cursor left on position found item else at end.
+ *                xmlrpc_queue cursor left on position found item else at end.
  *
  ***/
 
-int Q_Find(queue *q, void *data,
+int Q_Find(xmlrpc_queue *q, void *data,
            int (*Comp)(const void *, const void *))
 {
    void *d;
 
    if (q == NULL) {
-	return False_;
+    return False_;
    }
 
    d = Q_Head(q);
@@ -707,7 +706,7 @@ int Q_Find(queue *q, void *data,
    return False_;
 }
 
-/*========  Sorted Queue and Index functions   ========= */
+/*========  Sorted xmlrpc_queue and Index functions   ========= */
 
 
 static void QuickSort(void *list[], int low, int high,
@@ -753,9 +752,9 @@ static void QuickSort(void *list[], int low, int high,
  *
  ** function    : Q_Sort
  *
- ** purpose     : sort the queue and allow index style access.
+ ** purpose     : sort the xmlrpc_queue and allow index style access.
  *
- ** parameters  : queue pointer, comparison function compatible with
+ ** parameters  : xmlrpc_queue pointer, comparison function compatible with
  *                with 'qsort'.
  *
  ** returns     : True_ if sort succeeded. False_ if error occurred.
@@ -767,12 +766,12 @@ static void QuickSort(void *list[], int low, int high,
  *
  *                    for Comp(data1, data2)
  *
- *                If queue is already sorted it frees the memory of the
+ *                If xmlrpc_queue is already sorted it frees the memory of the
  *                old index and starts again.
  *
  ***/
 
-int Q_Sort(queue *q, int (*Comp)(const void *, const void *))
+int Q_Sort(xmlrpc_queue *q, int (*Comp)(const void *, const void *))
 {
    int         i;
    void        *d;
@@ -798,7 +797,7 @@ int Q_Sort(queue *q, int (*Comp)(const void *, const void *))
       return False_;
    }
 
-   /* Walk queue putting pointers into array */
+   /* Walk xmlrpc_queue putting pointers into array */
 
    d = Q_Head(q);
    for(i=0; i < q->size; i++) {
@@ -811,7 +810,7 @@ int Q_Sort(queue *q, int (*Comp)(const void *, const void *))
 
    QuickSort(index, 0, q->size - 1, Comp);
 
-   /* Rearrange the actual queue into correct order */
+   /* Rearrange the actual xmlrpc_queue into correct order */
 
    dn = q->head;
    i = 0;
@@ -836,9 +835,9 @@ int Q_Sort(queue *q, int (*Comp)(const void *, const void *))
  *
  ** function    : Q_BSearch
  *
- ** purpose     : binary search of queue index for node containing key
+ ** purpose     : binary search of xmlrpc_queue index for node containing key
  *
- ** parameters  : queue pointer 'q', data pointer of key 'key',
+ ** parameters  : xmlrpc_queue pointer 'q', data pointer of key 'key',
  *                  Comp comparison function.
  *
  ** returns     : integer index into array of node pointers,
@@ -848,7 +847,7 @@ int Q_Sort(queue *q, int (*Comp)(const void *, const void *))
  *
  ***/
 
-static int Q_BSearch( queue *q, void *key,
+static int Q_BSearch( xmlrpc_queue *q, void *key,
                       int (*Comp)(const void *, const void*))
 {
    int low, mid, hi, val;
@@ -882,22 +881,22 @@ static int Q_BSearch( queue *q, void *key,
  *
  ** purpose     : use index to locate data according to key in 'data'
  *
- ** parameters  : queue pointer 'q', data pointer 'data', Comp comparison
+ ** parameters  : xmlrpc_queue pointer 'q', data pointer 'data', Comp comparison
  *                function.
  *
  ** returns     : pointer to data or NULL if could not find it or could
- *                not sort queue.
+ *                not sort xmlrpc_queue.
  *
  ** comments    : see Q_Sort for description of 'Comp' function.
  *
  ***/
 
-void *Q_Seek(queue *q, void *data, int (*Comp)(const void *, const void *))
+void *Q_Seek(xmlrpc_queue *q, void *data, int (*Comp)(const void *, const void *))
 {
    int idx;
 
    if (q == NULL) {
-	return NULL;
+    return NULL;
    }
 
    if(!q->sorted) {
@@ -921,25 +920,25 @@ void *Q_Seek(queue *q, void *data, int (*Comp)(const void *, const void *))
  *
  ** function    : Q_Insert
  *
- ** purpose     : Insert an element into an indexed queue
+ ** purpose     : Insert an element into an indexed xmlrpc_queue
  *
- ** parameters  : queue pointer 'q', data pointer 'data', Comp comparison
+ ** parameters  : xmlrpc_queue pointer 'q', data pointer 'data', Comp comparison
  *                function.
  *
  ** returns     : pointer to data or NULL if could not find it or could
- *                not sort queue.
+ *                not sort xmlrpc_queue.
  *
  ** comments    : see Q_Sort for description of 'Comp' function.
  *                WARNING! This code can be very slow since each new
  *                element means a new Q_Sort.  Should only be used for
  *                the insertion of the odd element ,not the piecemeal
- *                building of an entire queue.
+ *                building of an entire xmlrpc_queue.
  ***/
 
-int Q_Insert(queue *q, void *data, int (*Comp)(const void *, const void *))
+int Q_Insert(xmlrpc_queue *q, void *data, int (*Comp)(const void *, const void *))
 {
    if (q == NULL) {
-	return False_;
+    return False_;
    }
 
    Q_PushHead(q, data);
@@ -950,12 +949,12 @@ int Q_Insert(queue *q, void *data, int (*Comp)(const void *, const void *))
    return True_;
 }
 
-/* read only funcs for iterating through queue. above funcs modify queue */
-q_iter Q_Iter_Head(queue *q) {
+/* read only funcs for iterating through xmlrpc_queue. above funcs modify xmlrpc_queue */
+q_iter Q_Iter_Head(xmlrpc_queue *q) {
    return q ? (q_iter)q->head : NULL;
 }
 
-q_iter Q_Iter_Tail(queue *q) {
+q_iter Q_Iter_Tail(xmlrpc_queue *q) {
    return q ? (q_iter)q->tail : NULL;
 }
 
