@@ -31,7 +31,7 @@
 */
 
 
-static const char rcsid[] = "#(@) $Id: xml_element.c,v 1.8 2002/05/23 17:46:51 danda Exp $";
+static const char rcsid[] = "#(@) $Id: xml_element.c,v 1.22 2008/04/16 13:03:15 robincornelius Exp $";
 
 
 
@@ -44,6 +44,48 @@ static const char rcsid[] = "#(@) $Id: xml_element.c,v 1.8 2002/05/23 17:46:51 d
  *   06/2000
  * HISTORY
  *   $Log: xml_element.c,v $
+ *   Revision 1.22  2008/04/16 13:03:15  robincornelius
+ *   tab2space the code, using 4 spaces for a tab
+ *
+ *   Revision 1.21  2008/04/14 12:05:49  robincornelius
+ *   remove libxml2 as the php compatability layer is a licence issue
+ *
+ *   Revision 1.20  2008/04/02 10:57:15  robincornelius
+ *   even more build warnings
+ *
+ *   Revision 1.19  2008/04/02 10:35:13  robincornelius
+ *   Fix a few more build warnings in expat and sample
+ *
+ *   Revision 1.18  2008/03/26 12:27:36  robincornelius
+ *   change build *again* to allow either expat or libxml as a parser
+ *
+ *   Revision 1.17  2008/03/26 08:59:35  robincornelius
+ *   better return value for file_out
+ *
+ *   Revision 1.16  2008/03/25 09:46:06  robincornelius
+ *   to fit in with autotools better, build private expat as default and enable shared expat with configure option
+ *
+ *   Revision 1.15  2008/03/24 15:24:09  robincornelius
+ *   allow optional building with builtin or system wide libexpat
+ *
+ *   Revision 1.14  2008/03/21 19:03:27  robincornelius
+ *   lets not use a private copy of libexpat
+ *
+ *   Revision 1.13  2008/03/21 17:37:09  robincornelius
+ *   fix strict gcc warnings some of which can cause arch specific issues, eg 64 bit failure
+ *
+ *   Revision 1.12  2008/03/21 09:59:10  robincornelius
+ *   fix a gcc error in build
+ *
+ *   Revision 1.11  2005/03/17 22:41:15  danda
+ *   committing Seth Nickell's <seth@gnome.org> patch submitted for IBM POWER contest.  Enables build on PPC 64.  Updates configure. Should help other platforms also.
+ *
+ *   Revision 1.10  2002/07/05 00:24:20  danda
+ *   merge in changes from php4 cvs
+ *
+ *   Revision 1.9  2002/07/03 20:54:30  danda
+ *   root element should not have a parent. patch from anon SF user
+ *
  *   Revision 1.8  2002/05/23 17:46:51  danda
  *   patch from mukund - fix non utf-8 encoding conversions
  *
@@ -89,12 +131,13 @@ static const char rcsid[] = "#(@) $Id: xml_element.c,v 1.8 2002/05/23 17:46:51 d
 #include <string.h>
 #include <ctype.h>
 
+#include <expat.h>
+
 #include "xml_element.h"
 #include "queue.h"
-#include "xmlparse.h"
 #include "encodings.h"
 
-#define my_free(thing)  if(thing) {free(thing); thing = 0;}
+#define my_free(thing)  if(thing) {free(thing); thing = NULL;}
 
 #define XML_DECL_START                 "<?xml"
 #define XML_DECL_START_LEN             sizeof(XML_DECL_START) - 1
@@ -170,7 +213,7 @@ void xml_elem_free_non_recurse(xml_element* root) {
 
       Q_Destroy(&root->children);
       Q_Destroy(&root->attrs);
-      my_free((char*)root->name);
+      my_free(root->name);
       simplestring_free(&root->text);
       my_free(root);
    }
@@ -296,13 +339,13 @@ static char* xml_elem_entity_escape(const char* buf, int old_len, int *newlen, X
     char *NewBuffer;
     int ToBeXmlEscaped=0;
     int iLength;
-    bufcopy = buf;
+    bufcopy = (const unsigned char *)buf;
     iLength= old_len ? old_len : strlen(buf);
     while(*bufcopy) {
       if( should_escape(*bufcopy, flags) ) {
-	/* the length will increase by length of xml escape - the character length */
-	iLength += entity_length(*bufcopy);
-	ToBeXmlEscaped=1;
+    /* the length will increase by length of xml escape - the character length */
+    iLength += entity_length(*bufcopy);
+    ToBeXmlEscaped=1;
       }
       bufcopy++;
     }
@@ -311,18 +354,18 @@ static char* xml_elem_entity_escape(const char* buf, int old_len, int *newlen, X
 
       NewBuffer= malloc(iLength+1);
       if(NewBuffer) {
-	bufcopy=buf;
-	while(*bufcopy) {
-	  if(should_escape(*bufcopy, flags)) {
-	    iNewBufLen += create_xml_escape(NewBuffer+iNewBufLen,*bufcopy);
-	  }
-	  else {
-	    NewBuffer[iNewBufLen++]=*bufcopy;
-	  }
-	  bufcopy++;
-	}
-	NewBuffer[iNewBufLen] = 0;
-	pRetval = NewBuffer;
+    bufcopy=(const unsigned char *)buf;
+    while(*bufcopy) {
+      if(should_escape(*bufcopy, flags)) {
+        iNewBufLen += create_xml_escape(NewBuffer+iNewBufLen,*bufcopy);
+      }
+      else {
+        NewBuffer[iNewBufLen++]=*bufcopy;
+      }
+      bufcopy++;
+    }
+    NewBuffer[iNewBufLen] = 0;
+    pRetval = NewBuffer;
       }
     }
   }
@@ -461,18 +504,19 @@ static void xml_element_serialize(xml_element *el, int (*fptr)(void *data, const
 }
 
 /* print buf to file */
-static file_out_fptr(void *f, const char *text, int size)
+static int file_out_fptr(void *f, const char *text, int size)
 {
-   fputs(text, (FILE *)f);
+   return fputs(text, (FILE *)f) >= 0 ? 0 : -1;
 }
 
 /* print buf to simplestring */
-static simplestring_out_fptr(void *f, const char *text, int size)
+static int simplestring_out_fptr(void *f, const char *text, int size)
 {
    simplestring* buf = (simplestring*)f;
    if(buf) {
       simplestring_addn(buf, text, size);
    }
+   return 0;
 }
 
 /****f* xml_element/xml_elem_serialize_to_string
@@ -552,18 +596,17 @@ typedef struct _xml_elem_data {
 
 
 /* expat start of element handler */
-static void startElement(void *userData, const char *name, const char **attrs)
+static void _xmlrpc_startElement(void *userData, const XML_Char *name, const XML_Char **attrs)
 {
-   int i;
    xml_element *c;
    xml_elem_data* mydata = (xml_elem_data*)userData;
-   const char** p = attrs;
+   const char** p = (const char**)attrs;
 
    if(mydata) {
       c = mydata->current;
 
       mydata->current = xml_elem_new();
-      mydata->current->name = (char*)strdup(name);
+      mydata->current->name = (char*)strdup((const char*)name);
       mydata->current->parent = c;
 
       /* init attrs */
@@ -572,7 +615,7 @@ static void startElement(void *userData, const char *name, const char **attrs)
          if(attr) {
             attr->key = strdup(*p);
             attr->val = strdup(*(p+1));
-            Q_PushTail(&mydata->current->attrs, attr);
+            Q_PushTail(&mydata->current->attrs, (const char**)attr);
 
             p += 2;
          }
@@ -581,7 +624,7 @@ static void startElement(void *userData, const char *name, const char **attrs)
 }
 
 /* expat end of element handler */
-static void endElement(void *userData, const char *name)
+static void _xmlrpc_endElement(void *userData, const XML_Char *name)
 {
    xml_elem_data* mydata = (xml_elem_data*)userData;
 
@@ -593,8 +636,8 @@ static void endElement(void *userData, const char *name)
 }
 
 /* expat char data handler */
-static void charHandler(void *userData,
-                        const char *s,
+static void _xmlrpc_charHandler(void *userData,
+                        const XML_Char *s,
                         int len)
 {
    xml_elem_data* mydata = (xml_elem_data*)userData;
@@ -603,7 +646,7 @@ static void charHandler(void *userData,
       /* Check if we need to decode utf-8 parser output to another encoding */
       if(mydata->needs_enc_conversion && mydata->input_options->encoding) {
          int new_len = 0;
-         char* add_text = utf8_decode(s, len, &new_len, mydata->input_options->encoding);
+         char* add_text = utf8_decode((const char*)s, len, &new_len, mydata->input_options->encoding);
          if(add_text) {
             len = new_len;
             simplestring_addn(&mydata->current->text, add_text, len);
@@ -611,7 +654,7 @@ static void charHandler(void *userData,
             return;
          }
       }
-      simplestring_addn(&mydata->current->text, s, len);
+      simplestring_addn(&mydata->current->text, (const char*)s, len);
    }
 }
 /******/
@@ -666,8 +709,8 @@ xml_element* xml_elem_parse_buf(const char* in_buf, int len, XML_ELEM_INPUT_OPTI
       mydata.input_options = options;
       mydata.needs_enc_conversion = options->encoding && strcmp(options->encoding, encoding_utf_8);
 
-      XML_SetElementHandler(parser, startElement, endElement);
-      XML_SetCharacterDataHandler(parser, charHandler);
+      XML_SetElementHandler(parser, _xmlrpc_startElement, _xmlrpc_endElement);
+      XML_SetCharacterDataHandler(parser, _xmlrpc_charHandler);
 
       /* pass the xml_elem_data struct along */
       XML_SetUserData(parser, (void*)&mydata);
@@ -677,17 +720,17 @@ xml_element* xml_elem_parse_buf(const char* in_buf, int len, XML_ELEM_INPUT_OPTI
       }
 
       /* parse the XML */
-      if(XML_Parse(parser, in_buf, len, 1) == 0) {
+      if(XML_Parse(parser, (const XML_Char *) in_buf, len, 1) == 0) {
          enum XML_Error err_code = XML_GetErrorCode(parser);
          int line_num = XML_GetCurrentLineNumber(parser);
          int col_num = XML_GetCurrentColumnNumber(parser);
          long byte_idx = XML_GetCurrentByteIndex(parser);
          int byte_total = XML_GetCurrentByteCount(parser);
-         const XML_LChar * error_str = XML_ErrorString(err_code);
+         const char * error_str = (const char *)XML_ErrorString(err_code);
          if(byte_idx >= 0) {
              snprintf(buf, 
                       sizeof(buf),
-                      "\n\tdata beginning %i before byte index: %s\n",
+                      "\n\tdata beginning %li before byte index: %s\n",
                       byte_idx > 10  ? 10 : byte_idx,
                       in_buf + (byte_idx > 10 ? byte_idx - 10 : byte_idx));
          }
@@ -696,7 +739,7 @@ xml_element* xml_elem_parse_buf(const char* in_buf, int len, XML_ELEM_INPUT_OPTI
                 "\tdescription: %s\n"
                 "\tline: %i\n"
                 "\tcolumn: %i\n"
-                "\tbyte index: %i\n"
+                "\tbyte index: %li\n"
                 "\ttotal bytes: %i\n%s ",
                 err_code, error_str, line_num, 
                 col_num, byte_idx, byte_total, buf);
@@ -713,6 +756,7 @@ xml_element* xml_elem_parse_buf(const char* in_buf, int len, XML_ELEM_INPUT_OPTI
       }
       else {
          xReturn = (xml_element*)Q_Head(&mydata.root->children);
+         xReturn->parent = NULL;
       }
 
       XML_ParserFree(parser);
