@@ -36,6 +36,12 @@
  *   Dan Libby, aka danda  (dan@libby.com)
  * HISTORY
  *   $Log: xmlrpc_introspection.c,v $
+ *   Revision 1.11  2008/03/21 17:37:09  robincornelius
+ *   fix strict gcc warnings some of which can cause arch specific issues, eg 64 bit failure
+ *
+ *   Revision 1.10  2005/03/17 22:33:22  danda
+ *   commit validation stuff.  untested
+ *
  *   Revision 1.9  2001/09/29 21:58:05  danda
  *   adding cvs log to history section
  *
@@ -61,6 +67,10 @@ static XMLRPC_VALUE xi_system_method_signature_cb(XMLRPC_SERVER server, XMLRPC_R
 static XMLRPC_VALUE xi_system_method_help_cb(XMLRPC_SERVER server, XMLRPC_REQUEST input, void* userData);
 
 
+/* forward declarations for (non public, non api) funcs */
+int validate_value( XMLRPC_VALUE xDesc, XMLRPC_VALUE xVal );
+
+
 /*-**********************************
 * Introspection Callbacks (methods) *
 ************************************/
@@ -340,7 +350,7 @@ XMLRPC_VALUE xml_element_to_method_description(xml_element* el, XMLRPC_ERROR err
          const char* ptype = !strcmp(el->name, "value") ? type : basetype;
          if(ptype) {
             if(Q_Size(&el->children) &&
-               !strcmp(ptype, "array") || !strcmp(ptype, "struct") || !strcmp(ptype, "mixed")) {
+               (!strcmp(ptype, "array") || !strcmp(ptype, "struct") || !strcmp(ptype, "mixed"))) {
                xSubList = XMLRPC_CreateVector("member", xmlrpc_vector_array);
 
                if(xSubList) {
@@ -376,7 +386,7 @@ XMLRPC_VALUE xml_element_to_method_description(xml_element* el, XMLRPC_ERROR err
 
       else if(!strcmp(el->name, "methodDescription")) {
          xml_element* elem_iter = Q_Head(&el->children);
-         xReturn = XMLRPC_CreateVector(NULL, xmlrpc_vector_struct);
+         xReturn = XMLRPC_CreateVector(name, xmlrpc_vector_struct);
 
          XMLRPC_VectorAppendString(xReturn, xi_token_name, name, 0);
 
@@ -592,4 +602,256 @@ int XMLRPC_ServerRegisterIntrospectionCallback(XMLRPC_SERVER server, XMLRPC_Intr
 ************************/
 
 
+/*-***********************************************
+* Request Validation Based on Introspection Data *
+*************************************************/
+
+int validate_scalar(XMLRPC_VALUE xDesc, XMLRPC_VALUE xVal, XMLRPC_VALUE_TYPE_EASY etype) {
+   int valid = 1;
+   const char* pmin = XMLRPC_VectorGetStringWithID(xDesc, xi_token_min);
+   const char* pmax = XMLRPC_VectorGetStringWithID(xDesc, xi_token_max);
+
+   if(etype == xmlrpc_int) {
+      int iVal = XMLRPC_GetValueInt(xVal);
+      if( pmin ) {
+         int min = atoi(pmin);
+         if( iVal < min ) {
+            valid = 0;
+         }
+      }
+      if( pmax ) {
+         int max = atoi(pmax);
+         if( iVal > max ) {
+            valid = 0;
+         }
+      }
+   }
+   else if(etype == xmlrpc_double) {
+      int dVal = XMLRPC_GetValueDouble(xVal);
+      if( pmin ) {
+         double min = atof(pmin);
+         if( dVal < min ) {
+            valid = 0;
+         }
+      }
+      if( pmax ) {
+         double max = atof(pmax);
+         if( dVal > max ) {
+            valid = 0;
+         }
+      }
+   }
+   else if(etype == xmlrpc_string ||
+           etype == xmlrpc_base64) {
+      int len = XMLRPC_GetValueStringLen(xVal);
+      if( pmin ) {
+         int min = atoi(pmin);
+         if( len < min ) {
+            valid = 0;
+         }
+      }
+      if( pmax ) {
+         int max = atoi(pmax);
+         if( len > max ) {
+            valid = 0;
+         }
+      }
+   }
+   return valid;
+}
+
+int validate_vector(XMLRPC_VALUE xDesc, XMLRPC_VALUE xVal, XMLRPC_VECTOR_TYPE vtype) {
+   XMLRPC_VALUE xSubDesc = XMLRPC_VectorGetValueWithID(xDesc, xi_token_sub);
+   int valid = 1;
+
+   /* check if its a struct */
+   if(vtype == xmlrpc_vector_struct) {
+      XMLRPC_VALUE xNextDesc = XMLRPC_VectorRewind(xSubDesc);
+      while( xNextDesc ) {
+         const char* name = XMLRPC_VectorGetStringWithID(xNextDesc, xi_token_name);
+         valid = validate_value( xNextDesc, XMLRPC_VectorGetValueWithID(xVal, name) );
+
+         if( !valid ) {
+            break;
+         }
+         xNextDesc = XMLRPC_VectorNext(xDesc);
+      }
+   }
+
+   /* not a struct. */
+   else {
+      XMLRPC_VALUE xNextDesc = XMLRPC_VectorRewind(xSubDesc);
+      int vsize = XMLRPC_VectorSize(xSubDesc);
+
+      XMLRPC_VALUE xNextVal = XMLRPC_VectorRewind(xVal);
+      do {
+         valid = validate_value( xNextDesc, xNextVal );
+         if( !valid)  {
+            break;
+         }
+         xNextVal = XMLRPC_VectorNext( xVal );
+
+         /* If just one description of an array type, then assume all
+          * elements should match that description/type
+          */
+         if( vsize != 1 ) {
+            xNextDesc = XMLRPC_VectorNext( xSubDesc );
+         }
+
+      } while ( xNextDesc && xNextVal );
+   }
+   return valid;
+}
+
+int validate_value( XMLRPC_VALUE xDesc, XMLRPC_VALUE xVal ) {
+   static const char* type_map[] = { "none",
+                                     "empty",
+                                     "base64",
+                                     "boolean",
+                                     "datetime",
+                                     "double",
+                                     "integer",
+                                     "string",
+                                     "array",
+                                     "mixed",
+                                     "struct" };
+
+   int valid = 0;
+
+   if( xDesc ) {
+      const char* type = XMLRPC_VectorGetStringWithID(xDesc, xi_token_type);
+      const char* opt = XMLRPC_VectorGetStringWithID(xDesc, xi_token_optional);
+      int optional = (opt && !strcmp(opt, "yes"));
+
+      if( !xVal ) {
+         valid = optional ? 1 : 0;
+      }
+      else {
+         XMLRPC_VALUE_TYPE_EASY etype = XMLRPC_GetValueTypeEasy(xVal);
+
+         /* make sure types match */
+         if( !strcmp( type_map[etype], type ) ) {
+            XMLRPC_VECTOR_TYPE vtype = XMLRPC_GetVectorType(xVal);
+
+            if( vtype == xmlrpc_vector_none) {
+               valid = validate_scalar(xDesc, xVal, etype);
+
+            }
+            else {
+               valid = validate_vector(xDesc, xVal, vtype);
+            }
+         }
+      }
+   }
+
+   return valid;
+}
+
+int validate_method(XMLRPC_VALUE xDesc, XMLRPC_VALUE xData) {
+   int valid = 1;
+   XMLRPC_VALUE xSigs = XMLRPC_VectorGetValueWithID(xDesc, xi_token_signatures);
+
+   XMLRPC_VALUE xNextSig = XMLRPC_VectorRewind(xSigs);
+
+   while(xNextSig) {
+      XMLRPC_VALUE xNextParam = XMLRPC_VectorRewind(xNextSig);
+
+      while( xNextParam ) {
+
+         if( !validate_value( xNextParam, xData ) ) {
+            valid = 0;
+            break;
+         }
+
+         xNextParam = XMLRPC_VectorNext( xNextSig );
+      }
+
+      xNextSig = XMLRPC_VectorNext( xSigs );
+   }
+
+   return valid;
+}
+
+
+/****f* SERVER/XMLRPC_ServerValidateRequest
+ * NAME
+ *   XMLRPC_ServerValidateRequest
+ * SYNOPSIS
+ *   int XMLRPC_ServerValidateRequest(XMLRPC_SERVER server, XMLRPC_REQUEST request, void* userData)
+ * FUNCTION
+ *   Validates that an XML-RPC request matches the signature defined by introspection data.
+ * INPUTS
+ *   server - target server
+ *   request - the request to be processed
+ *   userData - application specific data to be passed to introspection callbacks
+ * RESULT
+ *   int - 1 if valid request, else 0
+ * NOTES
+ *   untested
+ * SEE ALSO
+ *   XMLRPC_ServerSetValidationLevel ()
+ *   XMLRPC_IntrospectionCreateDescription ()
+ * SOURCE
+ */
+int XMLRPC_ServerValidateRequest(XMLRPC_SERVER server, XMLRPC_REQUEST request, void* userData) {
+
+   int valid = 1;
+
+   if( server && request ) {
+
+      if( server->validation != validation_none ) {
+
+         /* lazy loading of introspection data */
+         check_docs_loaded(server, userData);
+
+         if( server->xIntrospection && request->methodName.str ) {
+            XMLRPC_VALUE xIntMethod = XMLRPC_VectorGetValueWithID(server->xIntrospection, request->methodName.str );
+   
+            if( !xIntMethod ) {
+               if( server->validation == validation_all ) {
+                  valid = 0;
+               }
+            }
+            else {
+               valid = validate_method(xIntMethod, request->io);
+            }
+         }
+      }
+   }
+
+   return valid;
+
+}
+
+/****f* SERVER/XMLRPC_ServerSetValidationLevel
+ * NAME
+ *   XMLRPC_ServerSetValidationLevel
+ * SYNOPSIS
+ *   int XMLRPC_ServerSetValidationLevel(XMLRPC_SERVER server, XMLRPC_SERVER_VALIDATION level)
+ * FUNCTION
+ *   Sets the validation level
+ * INPUTS
+ *   server - target server
+ *   level - the validation level. ( none | all | if_defined )
+ * RESULT
+ *   int - 1 if success, else 0
+ * NOTES
+ *   untested
+ * SEE ALSO
+ *   XMLRPC_ServerValidateRequest ()
+ *   XMLRPC_IntrospectionCreateDescription ()
+ * SOURCE
+ */
+int XMLRPC_ServerSetValidationLevel(XMLRPC_SERVER server, XMLRPC_SERVER_VALIDATION level) {
+   int success = 0;
+   if(server) {
+      server->validation = level;
+      success = 1;
+   }
+   return success;
+}
+
+/*-***************************************************
+* End Request Validation Based on Introspection Data *
+*****************************************************/
 
